#ifndef DATA_H
#define DATA_H

#include <QString>
#include <QtSql/QSqlDatabase>
#include <QtSql/QSqlError>
#include <QtSql/QSqlQuery>
#include <QtSql/QSqlRecord>
#include <QMessageBox>
#include <QTextStream>
#include <QVariant>
#include <QDateTime>
#include <QDir>
#include <QIcon>
#include <QDebug>
#include <qmath.h>

/**The tdata class provides loading, storing and handling telemetric data.
 Exporting data as .sqlite or .csv is implemented in this class as well.
 */

class tdata : QObject
{
    public:
        tdata();
        tdata(QSqlDatabase database);
        ~tdata();

        int loadFromFile(QString path, QString name0);
        int exportToFile(bool csv, bool pictures, int beginning, int end, QString path);
        int exportCsv(int beginning, int end, QString path);
        int exportSqlite(int beginning, int end, QString path);
        int exportPictures(int beginning, int end, QString path);
        int initArrays(QSqlQuery query);

        int loadOBDData(QSqlQuery query);
        int loadSensorData(QSqlQuery query);
        int loadGPSData(QSqlQuery query);
        int loadUTCData(QSqlQuery query);

        /**Total length of the data.*/
        int length;
        /**Path to the .sqlite file.*/
        QString file_path;
        /**Name of the data.*/
        QString name;
        /**Database to handle querys.*/
        QSqlDatabase db;
        /**Query for database operations.*/
        QSqlQuery query;

        /**Stores the utc data.*/
        QStringList *utc;

        /**Stores the gps vehicle speed values.*/
        double *vehicle_speed;
        /**Stores the course values.*/
        double *course;

        /**Stores the x acceleration values.*/
        double *x_acceleration;
        /**Stores the y acceleration values.*/
        double *y_acceleration;
        /**Stores the z acceleration values.*/
        double *z_acceleration;

        /**Stores the gyroscope x acceleration values.*/
        double *x_acceleration_gyro;
        /**Stores the gyroscope y acceleration values.*/
        double *y_acceleration_gyro;
        /**Stores the gyroscope z acceleration values.*/
        double *z_acceleration_gyro;

        /**Stores the longitude values.*/
        double *longitude;
        /**Stores the latitude values.*/
        double *latitude;
        /**Stores the altitude values.*/
        double *altitude;

        /**Stores the obd vehicle speed values.*/
        double *vehicle_speed_obd;
        /**Stores the obd engine rpm values.*/
        double *engine_rpm_obd;
        /**Stores the obd throttle position.*/
        double *throttle_position_obd;
        /**Stores the obd barometric pressure values.*/
        double *barometric_pressure_obd;
        /**Stores the obd ambient air temperature values.*/
        double *ambient_air_temperature_obd;
        /**Stores the obd actual engine torque values.*/
        double *actual_engine_torque_obd;
        /**Stores the obd engine reference torque values.*/
        double *engine_reference_torque_obd;
        /**Stores the obd intake air temperature values.*/
        double *intake_air_temperature_obd;
        /**Stores the vehicle id.*/
        QString vid;
};

#endif // DATA_H
