#include "explorer.h"

/**Initializes the list widget and inserts available items.*/
explorer::explorer(QListWidget *qlist, thumbnailviewer *thumbs)
{
    //initialize listwidget
    listwidget = qlist;
    viewer = thumbs;

    //init db
    db = QSqlDatabase::addDatabase("QSQLITE");

    //init data
    data = new tdata(db);

    //create data directory
    dir = new QDir(NULL);

    //refresh
    refresh();
}

/**Adds an entry to the explorer.*/
int explorer::addEntry(QString name)
{
    //add to listwidget
    listwidget->addItem(name);

    return 0;
}

/**Refreshes the List Widget.*/
int explorer::refresh()
{
    //delete all items
    if(listwidget->count() > 0)
        listwidget->clear();

    //read available data
    //cd to data directory
    dir->cd(QDir::homePath());
    dir->cd("itlogger");

    //detect and sort available folders
    dir->setFilter(QDir::NoDot|QDir::NoDotDot|QDir::Dirs);
    QStringList ordner = dir->entryList();
    ordner.sort();

    //count folders and read all the data
    int count = ordner.count();
    for(int i = 0; i<count; i++)
    {
        //enter directory
        dir->cd(ordner.at(i));

        //TODO
        addEntry(ordner.at(i));

        //go back to data directory
        dir->cdUp();
    }

    return 0;
}

/**Deletes selected data.*/
int explorer::deleteEntry()
{
    //check if only one item is selected
    QList<QListWidgetItem*> selection = listwidget->selectedItems();
    QMessageBox *msg = new QMessageBox;


    //confirm deleting
    msg->setText(QObject::tr("Are you sure to delete this selection?"));
    msg->setWindowTitle(QObject::tr("Delete Entry"));
    msg->setIcon(QMessageBox::Question);
    msg->setWindowIcon(QIcon(":/images/icon.png"));
    msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
    msg->setButtonText( QMessageBox::Yes, QObject::tr("Yes"));
    msg->setButtonText( QMessageBox::No, QObject::tr("No"));

    //get item name
    QListWidgetItem* item;
    QDir dir;

    //yes/no
    switch(msg->exec())
    {
    case QMessageBox::Yes:
        //DELETE
        //delete data TODO
        for(int i=0;i<selection.count();i++){
            item=selection.at(i);
            dir.cd(dir.homePath());
            dir.cd("itlogger");
            dir.cd(item->text());

            Q_FOREACH(QFileInfo info, dir.entryInfoList(QDir::Files))
            {
                QFile::remove(info.absoluteFilePath());
            }

            if(dir.exists("DCIM"))
            {
                dir.cd("DCIM");
                Q_FOREACH(QFileInfo info, dir.entryInfoList(QDir::Files))
                {
                    QFile::remove(info.absoluteFilePath());
                }

                dir.cdUp();
                dir.rmdir("DCIM");
            }

            dir.cdUp();
            dir.rmdir(item->text());
        }
        //delete from listwidget
        qDeleteAll(selection);
        break;

    case QMessageBox::No:
        //!DELETE
        break;
    }

    delete msg;
    return 0;
}

/**Loads data and displays it.*/
bool explorer::load()
{
    //check if only one item is selected
    QList<QListWidgetItem*> selection = listwidget->selectedItems();
    QMessageBox msg;
    if(selection.count() != 1)
    {
        msg.setText(QObject::tr("Please select exactly one entry"));
        msg.setIcon(QMessageBox::Warning);
        msg.setWindowIcon(QIcon(":/images/icon.png"));
        msg.setWindowTitle(QObject::tr("Error"));
        msg.exec();
        return 0;
    }

    //get selected item
    QListWidgetItem* item = selection.at(0);
    dir->cd(QDir::homePath());
    dir->cd("itlogger");
    dir->cd(item->text());
    QString sourcepath;
    QStringList tables;
    sourcepath=dir->filePath("data.sqlite");


    //check if data exists
    if(dir->exists("data.sqlite"))
    {
        //check if database is compatible
        db.setDatabaseName(sourcepath);
        db.open();
        tables=db.tables();
        db.close();

        if(!tables.contains("OBD")||!tables.contains("GPS")||!tables.contains("SENSOREN"))
        {
            msg.setText(QObject::tr("Invalid database"));
            msg.setIcon(QMessageBox::Warning);
            msg.setWindowIcon(QIcon(":/images/icon.png"));
            msg.setWindowTitle(QObject::tr("Error"));
            msg.exec();
            return 0;
        }

        data->loadFromFile(dir->filePath("data.sqlite"), item->text());
    }

    else
    {
        //file not found
        msg.setText(QObject::tr("File not found"));
        msg.setIcon(QMessageBox::Warning);
        msg.setWindowIcon(QIcon(":/images/icon.png"));
        msg.setWindowTitle(QObject::tr("Error"));
        msg.exec();
        return false;
    }


    dir->cd("DCIM");
    viewer->loadImages(dir);

    return true;
}


/**Clones one set of data.*/
bool explorer::clone()
{
    //check if only one item is selected
    QList<QListWidgetItem*> selection = listwidget->selectedItems();
    QMessageBox *msg = new QMessageBox;
    if(selection.count() != 1)
    {
        msg->setText(QObject::tr("Please select exactly one entry"));
        msg->setIcon(QMessageBox::Warning);
        msg->setWindowIcon(QIcon(":/images/icon.png"));
        msg->setWindowTitle(QObject::tr("Error"));
        msg->exec();
        delete msg;
        return 0;
    }

    //get selected item
    QListWidgetItem* item = selection.at(0);
    dir->cd(QDir::homePath());
    dir->cd("itlogger");
    dir->cd(item->text());

    //check if data exists
    if(dir->exists("data.sqlite"))
    {
        QString filename;
        QString NewFilename;
        QString destinyPath;
        QString sourcePath;
        filename=item->text();
        int copycount=0;

        //go to data directory
        sourcePath=dir->absoluteFilePath("data.sqlite");

        dir->cdUp();

        //initialise NewFilename
        NewFilename= QString("%1_%2") .arg(filename) .arg(copycount);

        //check if copy already exists
        while(dir->exists(NewFilename))
        {
            //create new filename
            NewFilename= QString("%1_%2") .arg(filename) .arg(copycount);
            copycount++;
        }

        //create copy
        dir->mkdir(NewFilename);
        dir->cd(NewFilename);
        destinyPath=dir->absoluteFilePath("data.sqlite");

        //copy data.sqlite
        if(!file->copy(sourcePath,destinyPath))
        {
            //copy failed
            msg->setText(QObject::tr("Cloning failed"));
            msg->setIcon(QMessageBox::Warning);
            msg->setWindowIcon(QIcon(":/images/icon.png"));
            msg->setWindowTitle(QObject::tr("Error"));
            msg->exec();
            delete msg;
            return 0;
        }


        dir->cdUp();
        dir->cd(item->text());

        if(dir->exists("DCIM"))
        {
            //set destiny path
            dir->cdUp();
            dir->cd(NewFilename);
            dir->mkdir("DCIM");
            dir->cd("DCIM");
            destinyPath=dir->absolutePath();

            //go to source path
            dir->cdUp();
            dir->cdUp();
            dir->cd(item->text());
            dir->cd("DCIM");

            //copy pictures
            Q_FOREACH(QFileInfo info, dir->entryInfoList(QDir::Files))
            {
                file->copy(info.absoluteFilePath(),destinyPath+"/"+info.fileName());
            }
        }

        //refresh ListWidget
        refresh();
    }

    else
    {
        //file not found
        msg->setText(QObject::tr("File not found"));
        msg->setIcon(QMessageBox::Warning);
        msg->setWindowIcon(QIcon(":/images/icon.png"));
        msg->setWindowTitle(QObject::tr("Error"));
        msg->exec();
        delete msg;
        return false;
    }

    delete msg;
    return true;
}

/**Merges two .sqlite files.*/
bool explorer::merge()
{
    //check if only two items are selected
    QList<QListWidgetItem*> selection = listwidget->selectedItems();
    QMessageBox *msg = new QMessageBox;

    if(selection.count() != 2)
    {
        msg->setText(QObject::tr("Please select exactly two entries"));
        msg->setIcon(QMessageBox::Warning);
        msg->setWindowIcon(QIcon(":/images/icon.png"));
        msg->setWindowTitle(QObject::tr("Error"));
        msg->exec();
        delete msg;
        return 0;
    }

    //initialising
    QString convert; //help string
    QString UTC;
    QString filename;
    QString NewFilename;
    qulonglong time1end;
    qulonglong time1begin;
    qulonglong time2end;
    qulonglong time2begin;
    bool ok;
    int copycount=0;
    QString longitude;
    QString latitude;
    QSqlDatabase db1;
    QSqlDatabase db2;
    QListWidgetItem* item1 = selection.at(0);
    QListWidgetItem* item2 = selection.at(1);
    QDateTime time;

    //get selected item1
    dir->cd(dir->homePath()+"/itlogger/"+item1->text());

    //check if data exists
    if(dir->exists("data.sqlite"))
    {
        //init database
        db1 = QSqlDatabase::addDatabase("QSQLITE","db1");

        //open database
        db1.setDatabaseName(dir->filePath("data.sqlite"));
        db1.open();

        q = QSqlQuery(db1);

        q.prepare("SELECT UTC FROM OBD");
        q.exec();
        //read first UTC value
        q.first();
        UTC=q.value(0).toString();

        //get time
        convert=UTC.mid(0,4);
        convert.append(UTC.mid(5,2));
        convert.append(UTC.mid(8,2));
        convert.append(UTC.mid(11,2));
        convert.append(UTC.mid(14,2));
        convert.append(UTC.mid(17,2));
        convert.append(UTC.mid(20,3));
        time1begin=convert.toULongLong(&ok,10);

        //read last UTC value
        q.last();
        UTC=q.value(0).toString();

        //get time
        convert=UTC.mid(0,4);
        convert.append(UTC.mid(5,2));
        convert.append(UTC.mid(8,2));
        convert.append(UTC.mid(11,2));
        convert.append(UTC.mid(14,2));
        convert.append(UTC.mid(17,2));
        convert.append(UTC.mid(20,3));
        time1end=convert.toULongLong(&ok,10);

        //finish query
        q.finish();
        q.clear();
        //close database
        db1.close();
    }

    else
    {
        //file not found
        msg->setText(QObject::tr("File not found"));
        msg->setIcon(QMessageBox::Warning);
        msg->setWindowIcon(QIcon(":/images/icon.png"));
        msg->setWindowTitle(QObject::tr("Error"));
        msg->exec();
        delete msg;
        return false;
    }

    //get selected item2
    dir->cd(dir->homePath()+"/itlogger/"+item2->text());

    //check if data exists
    if(dir->exists("data.sqlite"))
    {
        //init database
        db2 = QSqlDatabase::addDatabase("QSQLITE","db2");

        //open database
        db2.setDatabaseName(dir->filePath("data.sqlite"));
        db2.open();

        q= QSqlQuery(db2);

        q.prepare("SELECT UTC FROM OBD");
        q.exec();

        //read first UTC value
        q.first();
        UTC=q.value(0).toString();

        //get time
        convert=UTC.mid(0,4);
        convert.append(UTC.mid(5,2));
        convert.append(UTC.mid(8,2));
        convert.append(UTC.mid(11,2));
        convert.append(UTC.mid(14,2));
        convert.append(UTC.mid(17,2));
        convert.append(UTC.mid(20,3));
        time2begin=convert.toULongLong(&ok,10);

        //read last UTC value
        q.last();
        UTC=q.value(0).toString();

        //get time
        convert=UTC.mid(0,4);
        convert.append(UTC.mid(5,2));
        convert.append(UTC.mid(8,2));
        convert.append(UTC.mid(11,2));
        convert.append(UTC.mid(14,2));
        convert.append(UTC.mid(17,2));
        convert.append(UTC.mid(20,3));
        time2end=convert.toULongLong(&ok,10);

        //finish query
        q.finish();
        q.clear();
        //close database
        db2.close();
    }

    else
    {
        //file not found
        msg->setText(QObject::tr("File not found"));
        msg->setIcon(QMessageBox::Warning);
        msg->setWindowIcon(QIcon(":/images/icon.png"));
        msg->setWindowTitle(QObject::tr("Error"));
        msg->exec();
        delete msg;
        return false;
    }

    if(time1begin<time2begin&&time1end+1<time2begin) //+1 because zero line is added later
    {
        //get selected item1
        dir->cd(dir->homePath()+"/itlogger/"+item1->text());

        //check if data exists
        if(dir->exists("data.sqlite"))
        {
            QString destinyPath;
            QString sourcePath;
            filename=item1->text();

            //go to data directory
            sourcePath=dir->absoluteFilePath("data.sqlite");

            dir->cdUp();

            //initialise NewFilename
            NewFilename= QString("%1_%2") .arg(filename) .arg(item2->text());

            //check if merged data already exists
            while(dir->exists(NewFilename))
            {
                //create new filename
                NewFilename= QString("%1_%2_%3") .arg(filename) .arg(item2->text()) .arg(copycount);
                copycount++;
            }

            //create merged file
            dir->mkdir(NewFilename);
            dir->cd(NewFilename);
            destinyPath=dir->absoluteFilePath("data.sqlite");

            //copy data.sqlite
            if(!file->copy(sourcePath,destinyPath))
            {
                //copy failed
                msg->setText(QObject::tr("Merging failed"));
                msg->setIcon(QMessageBox::Warning);
                msg->setWindowIcon(QIcon(":/images/icon.png"));
                msg->setWindowTitle(QObject::tr("Error"));
                msg->exec();
                delete msg;
                return 0;
            }

            dir->cd(dir->homePath()+"/itlogger/"+item1->text());

            if(dir->exists("DCIM"))
            {
                //set destiny path
                dir->cdUp();
                dir->cd(NewFilename);
                if(!dir->exists("DCIM"))dir->mkdir("DCIM");
                dir->cd("DCIM");
                destinyPath=dir->absolutePath();

                //go to source path item1
                dir->cd(dir->homePath()+"/itlogger/"+item1->text()+"/DCIM");

                //copy pictures
                Q_FOREACH(QFileInfo info, dir->entryInfoList(QDir::Files))
                {
                    file->copy(info.absoluteFilePath(),destinyPath+"/"+info.fileName());
                }
            }

            dir->cd(dir->homePath()+"/itlogger/"+item2->text());

            if(dir->exists("DCIM"))
            {
                //set destiny path
                dir->cdUp();
                dir->cd(NewFilename);
                if(!dir->exists("DCIM"))dir->mkdir("DCIM");
                dir->cd("DCIM");
                destinyPath=dir->absolutePath();

                //go to source path item2
                dir->cd(dir->homePath()+"/itlogger/"+item2->text());

                if(dir->exists("DCIM"))
                {
                    dir->cd("DCIM");
                    Q_FOREACH(QFileInfo info, dir->entryInfoList(QDir::Files))
                    {
                        file->copy(info.absoluteFilePath(),destinyPath+"/"+info.fileName());
                    }
                }
            }
        }
        //remember filename for sql queries
        filename=item2->text();
    }

    else if(time2begin<time1begin&&time2end+1<time1begin) //+1 because zero line is added later
    {
        //get selected item2
        dir->cd(dir->homePath()+"/itlogger/"+item2->text());

        //check if data exists
        if(dir->exists("data.sqlite"))
        {

            QString destinyPath;
            QString sourcePath;
            filename=item2->text();

            //go to data directory
            sourcePath=dir->absoluteFilePath("data.sqlite");

            dir->cdUp();

            //initialise NewFilename
            NewFilename= QString("%1_%2") .arg(filename) .arg(item1->text());

            //check if merged data already exists
            while(dir->exists(NewFilename))
            {
                //create new filename
                NewFilename= QString("%1_%2_%3") .arg(filename) .arg(item1->text()) .arg(copycount);
                copycount++;
            }

            //create merged file
            dir->mkdir(NewFilename);
            dir->cd(NewFilename);
            destinyPath=dir->absoluteFilePath("data.sqlite");

            //copy data.sqlite
            if(!file->copy(sourcePath,destinyPath))
            {
                //copy failed
                msg->setText(QObject::tr("Merging failed"));
                msg->setIcon(QMessageBox::Warning);
                msg->setWindowIcon(QIcon(":/images/icon.png"));
                msg->setWindowTitle(QObject::tr("Error"));
                msg->exec();
                delete msg;
                return 0;
            }

            dir->cd(dir->homePath()+"/itlogger/"+item1->text());

            if(dir->exists("DCIM"))
            {
                //set destiny path
                dir->cdUp();
                dir->cd(NewFilename);
                dir->mkdir("DCIM");
                dir->cd("DCIM");
                destinyPath=dir->absolutePath();

                //go to source path item1
                dir->cd(dir->homePath()+"/itlogger/"+item1->text()+"/DCIM");

                //copy pictures
                Q_FOREACH(QFileInfo info, dir->entryInfoList(QDir::Files))
                {
                    file->copy(info.absoluteFilePath(),destinyPath+"/"+info.fileName());
                }
            }

            dir->cd(dir->homePath()+"/itlogger/"+item2->text());

            if(dir->exists("DCIM"))
            {
                //set destiny path
                dir->cdUp();
                dir->cd(NewFilename);
                if(!dir->exists("DCIM"))dir->mkdir("DCIM");
                dir->cd("DCIM");
                destinyPath=dir->absolutePath();

                //go to source path item2
                dir->cd(dir->homePath()+"/itlogger/"+item2->text());

                if(dir->exists("DCIM"))
                {
                    dir->cd("DCIM");
                    Q_FOREACH(QFileInfo info, dir->entryInfoList(QDir::Files))
                    {
                        file->copy(info.absoluteFilePath(),destinyPath+"/"+info.fileName());
                    }
                }
            }
        }
        //remember filename for sql queries
        filename=item1->text();
    }
    else
    {
        msg->setText(QObject::tr("Data is overlapping"));
        msg->setIcon(QMessageBox::Warning);
        msg->setWindowIcon(QIcon(":/images/icon.png"));
        msg->setWindowTitle(QObject::tr("Error"));
        msg->exec();
        delete msg;
        return 0;
    }

    //open database
    dir->cd(dir->homePath()+"/itlogger/"+NewFilename);
    db.setDatabaseName(dir->filePath("data.sqlite"));

    db.open();

    //get current UTC time as key for database
    time = time.currentDateTimeUtc();
    UTC = time.toString("yyyyMMddhhmmsszzz");

    //get last longitude, latitude
    q = QSqlQuery(db);

    q.prepare("SELECT Longitude from GPS");
    q.exec();
    q.last();
    longitude = q.value(0).toString();
    q.finish();

    q.prepare("SELECT Latitude from GPS");
    q.exec();
    q.last();
    latitude = q.value(0).toString();
    q.finish();

    //insert zeros
    q.prepare("INSERT INTO OBD (UTC,VehicleID,EngineRPM,VehicleSpeed,ThrottlePosition,"
              "BarometricPressure,AmbientAirTemperature,ActualEnginePercentTorque,"
              "EngineReferenceTorque,IntakeAirTemperature)"
              "VALUES ("+UTC+",0,0,0,0,0,0,0,0,0)");
    q.exec();
    q.finish();

    q.prepare("INSERT INTO GPS (UTC,Longitude,Latitude,Altitude,Course,Speed)"
              "VALUES ("+UTC+","+longitude+","+latitude+",0,0,0)");
    q.exec();
    q.finish();

    q.prepare("INSERT INTO SENSOREN (UTC,XAcceleration,YAcceleration,ZAcceleration,"
              "XGyroscope,YGyroscope,ZGyroscope) VALUES ("+UTC+",0,0,0,0,0,0)");
    q.exec();
    q.finish();

    //attach database db2 and merge with db1
    dir->cd(dir->homePath()+"/itlogger/"+filename);
    convert=dir->toNativeSeparators(dir->filePath("data.sqlite"));
    q.prepare("ATTACH DATABASE '" + convert + "' AS toMerge");
    q.exec();
    q.finish();

    q.prepare("INSERT INTO OBD SELECT * FROM toMerge.OBD");
    q.exec();
    q.finish();

    q.prepare("INSERT INTO GPS SELECT * FROM toMerge.GPS");
    q.exec();
    q.finish();

    q.prepare("INSERT INTO SENSOREN SELECT * FROM toMerge.SENSOREN");
    q.exec();
    q.finish();

    q.prepare("DETACH DATABASE toMerge");
    q.exec();
    q.finish();

    q.clear();
    db.close();

    //refresh List Widget
    refresh();


    delete msg;
    return true;
}
