#include "importhandler.h"

/**Constructs the importhandler.*/
importhandler::importhandler()
{
    //init
    dir = new QDir(NULL);
    file = new QFile;
    db = QSqlDatabase::addDatabase("QSQLITE","db");
    importFromDrives();
}


/**Imports data from sd card or similar devices.*/
int importhandler::importFromDrives()
{
    //init
    bool importing = false;
    QMessageBox *msg = new QMessageBox;
    QStringList tables;
    QDir hompath = (QDir::homePath()+"/itlogger");

    //get available drives
    QSystemStorageInfo *storage = new QSystemStorageInfo;
    QStringList list = storage->logicalDrives();

    //check if drive is removable storage
    for(int i = 0; i<list.count(); i++)
    {

        if(storage->typeForDrive(list.at(i)) == QSystemStorageInfo::RemovableDrive || storage->typeForDrive(list.at(i)) == QSystemStorageInfo::InternalFlashDrive)
        {
            QString devicepath = (list.at(i)+"/itlogger");
            QDir devicedir(devicepath);

            if(devicedir.exists())
            {
                //write dirs from removable storage in a list
                QStringList devicedirs = devicedir.entryList(QDir::NoDotAndDotDot|QDir::Dirs|QDir::NoSymLinks|QDir::Readable);

                //list with dir names from homepath
                QStringList homedirs = hompath.entryList(QDir::NoDotAndDotDot|QDir::Dirs|QDir::NoSymLinks|QDir::Readable);

                //copied as many times as there are folders
                for (int j = 0; j  < devicedirs.count(); j++)
                {
                    QString sourcepath;
                    sourcepath = devicepath + "/" + devicedirs[j] + "/data.sqlite";

                    //check if data already exists
                    if(homedirs.contains(devicedirs.at(j)))
                        continue;

                    //no data selected
                    if(sourcepath.isNull())
                        continue;

                    //check if database is compatible
                    db.setDatabaseName(sourcepath);
                    db.open();
                    tables=db.tables();
                    db.close();

                    if(!tables.contains("OBD")||!tables.contains("GPS")||!tables.contains("SENSOREN"))
                        continue;

                    //set directories
                    QString path = sourcepath;
                    QString destinypath;
                    path.chop(12);
                    dir->setPath(path);
                    QString filename=dir->dirName();
                    dir->cd(dir->homePath()+"/itlogger");

                    if(!importing)
                    {
                        importing = true;
                        QMessageBox *msg = new QMessageBox;
                        msg->setText(QObject::tr("New data found! Would you like to import it?"));
                        msg->setWindowTitle(QObject::tr("Import Data"));
                        msg->setIcon(QMessageBox::Question);
                        msg->setWindowIcon(QIcon(":/images/icon.png"));
                        msg->setStandardButtons(QMessageBox::Yes | QMessageBox::No);
                        msg->setButtonText(QMessageBox::Yes, QObject::tr("Yes"));
                        msg->setButtonText(QMessageBox::No, QObject::tr("No"));

                        switch(msg->exec())
                        {
                        case QMessageBox::Yes:
                            //create directory
                            dir->mkdir(filename);
                            dir->cd(filename);
                            destinypath=dir->absoluteFilePath("data.sqlite");

                            //import data.sqlite
                            if(!file->copy(sourcepath,destinypath))
                            {
                                //import failed
                                msg->setText(QObject::tr("Import failed"));
                                msg->setIcon(QMessageBox::Warning);
                                msg->setWindowIcon(QIcon(":/images/icon.png"));
                                msg->setWindowTitle(QObject::tr("Error"));
                                msg->exec();
                                delete msg;
                                return 0;
                            }

                            dir->cd(path);
                            if(dir->exists("DCIM"))
                            {
                                //set destiny path
                                dir->cd(dir->homePath()+"/itlogger");
                                dir->cd(filename);
                                dir->mkdir("DCIM");
                                dir->cd("DCIM");
                                destinypath=dir->absolutePath();

                                //go to source path
                                dir->cd(path);
                                dir->cd("DCIM");

                                //copy pictures
                                Q_FOREACH(QFileInfo info, dir->entryInfoList(QDir::Files))
                                {
                                    file->copy(info.absoluteFilePath(),destinypath+"/"+info.fileName());
                                }
                            }

                            break;

                        case QMessageBox::No:
                            delete msg;
                            return 0;
                        }
                    }

                    else
                    {
                        //create directory
                        dir->mkdir(filename);
                        dir->cd(filename);
                        destinypath=dir->absoluteFilePath("data.sqlite");

                        //import data.sqlite
                        if(!file->copy(sourcepath,destinypath))
                        {
                            //import failed
                            msg->setText(QObject::tr("Import failed"));
                            msg->setIcon(QMessageBox::Warning);
                            msg->setWindowIcon(QIcon(":/images/icon.png"));
                            msg->setWindowTitle(QObject::tr("Error"));
                            msg->exec();
                            delete msg;
                            return 0;
                        }

                        dir->cd(path);
                        if(dir->exists("DCIM"))
                        {
                            //set destiny path
                            dir->cd(dir->homePath()+"/itlogger");
                            dir->cd(filename);
                            dir->mkdir("DCIM");
                            dir->cd("DCIM");
                            destinypath=dir->absolutePath();

                            //go to source path
                            dir->cd(path);
                            dir->cd("DCIM");

                            //copy pictures
                            Q_FOREACH(QFileInfo info, dir->entryInfoList(QDir::Files))
                            {
                                file->copy(info.absoluteFilePath(),destinypath+"/"+info.fileName());
                            }
                        }
                    }
                }
            }
        }
    }
    return 0;
}

/**Imports data from a file.*/
int importhandler::importFromFile()
{
    //init
    QMessageBox *msg = new QMessageBox;
    QStringList tables;
    int count = 0;
    QString sourcepath = QFileDialog::getOpenFileName(NULL, QObject::tr("Open File"), "",QObject::tr("Files (*.sqlite)"));


    //no data selected
    if (sourcepath.isNull()) return 0;

    //check if database is compatible
    db.setDatabaseName(sourcepath);
    db.open();
    tables=db.tables();
    db.close();

    if(!tables.contains("OBD")||!tables.contains("GPS")||!tables.contains("SENSOREN"))
    {
        msg->setText(QObject::tr("Invalid database"));
        msg->setIcon(QMessageBox::Warning);
        msg->setWindowIcon(QIcon(":/images/icon.png"));
        msg->setWindowTitle(QObject::tr("Error"));
        msg->exec();
        delete msg;
        return 0;
    }

    //set directories
    QString path = sourcepath;
    QString destinypath;
    path.chop(12);
    dir->setPath(path);
    QString filename=dir->dirName();
    dir->cd(dir->homePath()+"/itlogger");

    //check if data already exists
    while(dir->exists(filename))
    {
        //create new filename
        filename = QString("%1_%2") .arg(filename) .arg(count);
        count++;
    }

    //create directory
    dir->mkdir(filename);
    dir->cd(filename);
    destinypath=dir->absoluteFilePath("data.sqlite");

    //import data.sqlite
    if(!file->copy(sourcepath,destinypath))
    {
        //import failed
        msg->setText(QObject::tr("Import failed"));
        msg->setIcon(QMessageBox::Warning);
        msg->setWindowIcon(QIcon(":/images/icon.png"));
        msg->setWindowTitle(QObject::tr("Error"));
        msg->exec();
        delete msg;

    }

    dir->cd(path);
    if(dir->exists("DCIM"))
    {
        //set destiny path
        dir->cd(dir->homePath()+"/itlogger");
        dir->cd(filename);
        dir->mkdir("DCIM");
        dir->cd("DCIM");
        destinypath=dir->absolutePath();

        //go to source path
        dir->cd(path);
        dir->cd("DCIM");

        //copy pictures
        Q_FOREACH(QFileInfo info, dir->entryInfoList(QDir::Files))
        {
            file->copy(info.absoluteFilePath(),destinypath+"/"+info.fileName());
        }
    }

    return 0;
}

