/*Copyright (C) 2012 Johannes Hiller, Michael Hoppe, Waldemar Heinrich
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*   you may not use this file except in compliance with the License.
*   You may obtain a copy of the License at
*
*       http://www.apache.org/licenses/LICENSE-2.0
*
*   Unless required by applicable law or agreed to in writing, software
*   distributed under the License is distributed on an "AS IS" BASIS,
*   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*   See the License for the specific language governing permissions and
*   limitations under the License.
*/

package de.tui.itlogger;


import java.text.SimpleDateFormat;
import java.util.Calendar;
import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;

/**Class for writing data in a SQLite database*/
public class SQLite extends iTloggerActivity
{		 
	private static final String TABLE_OBD = "OBD";
	private static final String TABLE_GPS = "GPS";
	private static final String TABLE_SENSOREN = "SENSOREN";
	static boolean ready=true;
	static SQLiteDatabase db = null;
		        
	/**Initializes the tables and writes the values */
	 static void onCreateDBAndDBTabled()
	    {
	    	try {
	    		//Create database and tables
	    		db = SQLiteDatabase.openOrCreateDatabase(paths+iTloggerActivity.dbnamefolder+"_"+iTloggerActivity.formattedDate+"/"+"data.sqlite", null);
	    		Cursor c1 = db.rawQuery("PRAGMA journal_mode = MEMORY", null); c1.close();
	    		/**Creates the table for the OBD2 values*/
	    		db.execSQL("CREATE TABLE IF NOT EXISTS " + TABLE_OBD + " (UTC time(21) primary key, "
	    															   +"VehicleID string(17), "
	    															   +"EngineRPM integer(5), "
	    															   +"VehicleSpeed integer(3), "
	    															   +"ThrottlePosition integer(3), "
	    															   +"BarometricPressure integer(3), "
	    															   +"AmbientAirTemperature integer(3), "
	    															   +"ActualEnginePercentTorque integer(3), "
	    															   +"EngineReferenceTorque integer(3), "
	    															   +"IntakeAirTemperature integer(3))"
	    															   +";");
	    		
	    		//Creates the table for the GPS values
	    		db.execSQL("CREATE TABLE IF NOT EXISTS " + TABLE_GPS + " (UTC time(21) primary key, "
	    															   +"Longitude integer(7), "
	    															   +"Latitude integer(7), "
	    															   +"Altitude integer(5), "
	    															   +"Course integer(4), "
	    															   +"Speed integer(5))"
	    															   +";");	
	    				
	    		//Creates the table for the sensor values
	    		db.execSQL("CREATE TABLE IF NOT EXISTS " + TABLE_SENSOREN + " (UTC time(21) primary key, "
						   													+"XAcceleration integer(3), "
						   													+"YAcceleration integer(3), "
						   													+"ZAcceleration integer(3), "
						   													+"XGyroscope integer(3), "
						   													+"YGyroscope integer(3), "
						   													+"ZGyroscope integer(3))"
						   													+";");	  					
	        } finally{
	        	db.close();	    	 
	        }
	    }
	   	static void writeDB() {

	   		ready=false;
    				 //Read the UTC time from the device
	            	 
	            	 Calendar c = Calendar.getInstance(); 
	            	 SimpleDateFormat dtf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss:SSS"); 
	            	 String formattedDates = dtf.format(c.getTime());
 
	            //Write data in OBD table
  	        	ContentValues insertValuesOBD = new ContentValues(); 
	            insertValuesOBD.put("UTC", formattedDates); 
	            if((OBD2_Activity.selected[8] == true) && (OBD2_Activity.VID != "-")){
  	        	insertValuesOBD.put("VehicleID", OBD2_Activity.VID);
	            }
	            if((OBD2_Activity.selected[0] == true) && (OBD2_Activity.OBD_values[0] != -255)){
  	        	insertValuesOBD.put("EngineRPM", OBD2_Activity.OBD_values[0]);
	            }
	            if((OBD2_Activity.selected[1] == true) && (OBD2_Activity.OBD_values[1] != -255)){
  	        	insertValuesOBD.put("VehicleSpeed", OBD2_Activity.OBD_values[1]);
	            }
	            if((OBD2_Activity.selected[2] == true) && (OBD2_Activity.OBD_values[2] != -255)){
  	        	insertValuesOBD.put("ThrottlePosition", OBD2_Activity.OBD_values[2]);
	            }
	            if((OBD2_Activity.selected[3] == true) && (OBD2_Activity.OBD_values[3] != -255)){
  	        	insertValuesOBD.put("BarometricPressure", OBD2_Activity.OBD_values[3]);
	            }
	            if((OBD2_Activity.selected[4] == true) && (OBD2_Activity.OBD_values[4] != -255)){
  	        	insertValuesOBD.put("AmbientAirTemperature", OBD2_Activity. OBD_values[4]);
	            }
	            if((OBD2_Activity.selected[5] == true) && (OBD2_Activity.OBD_values[5] != -255)){
  	        	insertValuesOBD.put("ActualEnginePercentTorque", OBD2_Activity.OBD_values[5]);
	            }
	            if((OBD2_Activity.selected[6] == true) && (OBD2_Activity.OBD_values[6] != -255)){
  	        	insertValuesOBD.put("EngineReferenceTorque", OBD2_Activity.OBD_values[6]);
	            }
	            if((OBD2_Activity.selected[7] == true) && (OBD2_Activity.OBD_values[7] != -255)){
  	        	insertValuesOBD.put("IntakeAirTemperature", OBD2_Activity.OBD_values[7]);
	            }
  	        	db.insert(TABLE_OBD, null, insertValuesOBD);
	     	        
  	        	//Write data in GPS table
  	        	ContentValues insertValuesGPS = new ContentValues(); 
	            insertValuesGPS.put("UTC", formattedDates); 
	            insertValuesGPS.put("Longitude", GPS.gps_data[0]);
  	        	insertValuesGPS.put("Latitude", GPS.gps_data[1]);
  	        	insertValuesGPS.put("Speed", GPS.gps_data[2]);
  	        	insertValuesGPS.put("Altitude", GPS.gps_data[3]);
  	        	insertValuesGPS.put("Course", GPS.gps_data[4]);
  	        	
  	        	db.insert(TABLE_GPS, null, insertValuesGPS);
  	        	
  	        	//Write data in SENSOREN table
  	        	ContentValues insertValuesSENSOREN = new ContentValues(); 
	            insertValuesSENSOREN.put("UTC", formattedDates); 
  	        	insertValuesSENSOREN.put("XAcceleration", Sensoren.values_a[0]);
  	        	insertValuesSENSOREN.put("YAcceleration", Sensoren.values_a[1]);
  	        	insertValuesSENSOREN.put("ZAcceleration", Sensoren.values_a[2]);
  	        	insertValuesSENSOREN.put("XGyroscope", Sensoren.values_g[0]);
  	        	insertValuesSENSOREN.put("YGyroscope", Sensoren.values_g[1]);
  	        	insertValuesSENSOREN.put("ZGyroscope", Sensoren.values_g[2]);
	            db.insert(TABLE_SENSOREN, null, insertValuesSENSOREN);
	            
	            ready=true;      
	   	}
}