/*Copyright (C) 2012 Johannes Hiller, Michael Hoppe, Waldemar Heinrich
*
*  Licensed under the Apache License, Version 2.0 (the "License");
*   you may not use this file except in compliance with the License.
*   You may obtain a copy of the License at
*
*       http://www.apache.org/licenses/LICENSE-2.0
*
*   Unless required by applicable law or agreed to in writing, software
*   distributed under the License is distributed on an "AS IS" BASIS,
*   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
*   See the License for the specific language governing permissions and
*   limitations under the License.
*/

package de.tui.itlogger;

import java.io.File;
import java.util.List;

import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager.NameNotFoundException;
import android.hardware.Camera;
import android.hardware.Camera.Size;
import android.os.Bundle;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.util.Log;
import android.widget.TextView;
import android.widget.Toast;

import com.actionbarsherlock.app.SherlockActivity;

/**
 * Provides the splash screen at the startup of the app and polls the
 * camera resolutions in order to save time when going to preferences
 */
public class SplashActivity extends SherlockActivity {

	private Camera mCamera;
	static String path = "/sdcard/";
	static String appname; 

	/**
	 * Sets the layout and calls {@link displayVersionName} and {@link getResolutions}
	 * @param savedInstanceState gets any saved preferences
	 */
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		// TODO Auto-generated method stub
		super.onCreate(savedInstanceState);
		setContentView(R.layout.splash);
		displayVersionName();
		Context context = getApplicationContext();
		SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(context);
	    SharedPreferences.Editor editor = settings.edit();
	    editor.putBoolean("toggleobd2", false);
	    editor.putBoolean("calibrate", false);  
	    editor.putBoolean("calibrated", false);
	    editor.putBoolean("nameset", false);
	    editor.putString("savefolder", "");
	    editor.commit();
	    appname=getString(R.string.app_name);
		(new File(path + appname)).mkdir();
		
	    String extState = Environment.getExternalStorageState();
	    //you may also want to add (...|| Environment.MEDIA_MOUNTED_READ_ONLY)
	    //if you are only interested in reading the filesystem
	    if(extState.equals(Environment.MEDIA_MOUNTED)) {
	    	Thread timer = new Thread() {
				public void run() {
					try {
						getResolutions();
						sleep(2000);// 2 seconds
					} catch (InterruptedException e) {
						e.printStackTrace();
					} finally {
						//starts main activity
						Intent intent = new Intent(SplashActivity.this, iTloggerActivity.class);
						startActivity(intent);
					}
				}
			};
			timer.start();	    }
	    else {
	    	//Displayed if no SD card is mounted
	    	String nosdcard = getString(R.string.nosdcard);
			int duration = Toast.LENGTH_LONG;
			Toast.makeText(context, nosdcard, duration).show();
			finish();
	    }
	}
	/**
	 * Finishes the activity on pause
	 */
	@Override
	protected void onPause() {
		// TODO Auto-generated method stub
		super.onPause();
		finish();
	}
	
	/**
	 * Gets the version of the app and writes in a textview
	 */
	public void displayVersionName() {
	    String versionName = "";
	    PackageInfo packageInfo;
	    try {
	        packageInfo = getPackageManager().getPackageInfo(getPackageName(), 0);
	    	versionName = "v " + packageInfo.versionName;
	    } catch (NameNotFoundException e) {
	        e.printStackTrace();
	    }
	    TextView tv = (TextView) findViewById(R.id.appversion);
	    tv.setText(versionName);
	}
	/**
	 * Gets the resolutions of the camera and divides them into 3 parts, takes lowest and
	 * highest resolution and the resolution at 1/3 and 2/3 of the length of the
	 * resolution-array and writes them into an array for the preferences
	 * Turns the array around, depending on where the highest value is found
	 * Improves startup time of iTloggerPreferences
	 */
	public void getResolutions() {
		mCamera = getCameraInstance();
        Camera.Parameters params = mCamera.getParameters();
        List sizes = params.getSupportedPictureSizes();
        Camera.Size availablesizes = null;
        int j=sizes.size()-1;
        Camera.Size maxvalue =(Size) sizes.get(j);
        Camera.Size minvalue=(Size) sizes.get(0);
        //If the highest resolution is at the end of the array
        if (minvalue.height<=maxvalue.height) {
            for (int i=0;i<sizes.size();i++){
                availablesizes = (Size) sizes.get(i);
                
                //Takes the resolution at the beginning and sets it to the lowest resolution
                if (i<=0) {
                	iTloggerPreferences.lowwidth=availablesizes.width;
                	iTloggerPreferences.w=availablesizes.width;
                	iTloggerPreferences.lowheight=availablesizes.height;
                	iTloggerPreferences.h=availablesizes.height;
	               
	                String minres = getString(R.string.lowrestext);
	                iTloggerPreferences.resolutions[0]=(minres+" "+iTloggerPreferences.lowwidth+"x"+iTloggerPreferences.lowheight);
                }
                //Takes the resolution at the end and sets it to maximum resolution
                if (i>=j-1){
                	iTloggerPreferences.maxwidth=availablesizes.width;
                	iTloggerPreferences.maxheight=availablesizes.height;
	               
	                String maxres = getString(R.string.maxrestext);
	                iTloggerPreferences.resolutions[3]=(maxres+" "+iTloggerPreferences.maxwidth+"x"+iTloggerPreferences.maxheight);
                }
                //Takes the resolution at 1/3 and sets it to middle resolution
                if (i<=j/3 && i>=j/3) {
                	iTloggerPreferences.middlewidth=availablesizes.width;
                	iTloggerPreferences.middleheight=availablesizes.height;
	                
	                String middleres = getString(R.string.middlerestext);
	                iTloggerPreferences.resolutions[1]=(middleres+" "+iTloggerPreferences.middlewidth+"x"+iTloggerPreferences.middleheight);
                }
                //Takes the resolution at 2/3 and sets it to high resolution
                if (i<=2*j/3 && i>=2*j/3){
                	iTloggerPreferences.highwidth=availablesizes.width;
                	iTloggerPreferences.highheight=availablesizes.height;
	               
	                String highres = getString(R.string.highrestext);
	                iTloggerPreferences.resolutions[2]=(highres+" "+iTloggerPreferences.highwidth+"x"+iTloggerPreferences.highheight);
                }
            }
            //If the highest resolution is at the beginning of the array
        } else {
        	for (int i=0;i<sizes.size();i++){
                availablesizes = (Size) sizes.get(i);
               
                //Takes the resolution at the beginning and sets it to maximum resolution
                if (i<=0) {
                	iTloggerPreferences.maxwidth=availablesizes.width;
                	iTloggerPreferences.maxheight=availablesizes.height;
	               
	                String maxres = getString(R.string.maxrestext);
	                iTloggerPreferences.resolutions[3]=(maxres+" "+iTloggerPreferences.maxwidth+"x"+iTloggerPreferences.maxheight);
                }
                //Takes the resolution at the beginning and sets it to the lowest resolution
                if (i>=j-1){
                	iTloggerPreferences.lowwidth=availablesizes.width;
                	iTloggerPreferences.w=availablesizes.width;
                	iTloggerPreferences.lowheight=availablesizes.height;
                	iTloggerPreferences.h=availablesizes.height;
	                
	                String minres = getString(R.string.lowrestext);
	                iTloggerPreferences.resolutions[0]=(minres+" "+iTloggerPreferences.lowwidth+"x"+iTloggerPreferences.lowheight);
                }
                //Takes the resolution at 1/3 and sets it to high resolution
                if (i<=j/3 && i>=j/3) {
                	iTloggerPreferences.highwidth=availablesizes.width;
                	iTloggerPreferences.highheight=availablesizes.height;
	               
	                String highres = getString(R.string.highrestext);
	                iTloggerPreferences.resolutions[2]=(highres+" "+iTloggerPreferences.highwidth+"x"+iTloggerPreferences.highheight);
                }
                //Takes the resolution at 2/3 and sets it to middle resolution
                if (i<=2*j/3 && i>=2*j/3){
	                iTloggerPreferences.middlewidth=availablesizes.width;
                	iTloggerPreferences.middleheight=availablesizes.height;
	                
	                String middleres = getString(R.string.middlerestext);
	                iTloggerPreferences.resolutions[1]=(middleres+" "+iTloggerPreferences.middlewidth+"x"+iTloggerPreferences.middleheight);
                }
            }
        }
            mCamera.release();
	}
	/**
	 * Gets an instance of the camera for getting the supported resoutions
	 */
	public static Camera getCameraInstance(){
	    Camera c = null;
	    try {
	        c = Camera.open(); // attempt to get a Camera instance
	    }
	    catch (Exception e){
	        // Camera is not available (in use or does not exist)
	    }
	    return c; // returns null if camera is unavailable
	}
}